#!/usr/bin/perl -w
#############################################################################
## Name:        WordFlashReader_018.pl
## Author:      Winship Varner
## Created:     11/02/2001
## Modified:	05/12/2002
## Copyright:   (c) 2001-2002 Winship Varner
## Licence:     This program is free software; you can redistribute it and/or
##              modify it in accordance with the GNU General Public Licence.
#############################################################################
use Tk;
require Tk::TextUndo;
require Tk::ROText;
use strict;
use Cwd;
############################################################
## Variable Definitions
############################################################

my $CHARMAX = 1;

my $linesave ="";
my @configuration = (); # break up the ini file into usable chunks
my $check_save = 1; # reminder before shutting down unsaved files
my $end = 0; # index of the last line of $file
my $font_color = "yellow";
my $font_type = "times";
my $font_size = "32";
my $home = $ENV{HOME};
my $default_dir = $ENV{HOME}; # directory last opened
my $speed = "91"; # speed of word display
my $file = ""; # /dir/to/file_opened.txt
if($ARGV[0]){$file = "$ARGV[0]"} # open file from command line
my @words = (); # $line[$word_num]
my $word = ""; # the individual word that is displayed
my $word_num = 0;
my $line = ""; # set from each line of the file opened
my $line_num = 1; # line index
# $line_begin and $line_end are character indexes
my $line_begin = 0.0;
my $line_end = 0;
my $line_index1 = 0;
my $line_index2 = 0;
my $punct = ""; # used to determine what punctuation a word has
my @recent_docs = (); # recently opened document set from dot file
# $status Keeps track of what has/is happing in the program. The
# options are: not_loaded, loaded, started, paused, saved, ended
my $status = "not_loaded";

############# read from the dot file - set personalized varables
ini_read();

##################################################################
## GUI Interface
##################################################################
my $main = new MainWindow;
my $menu = $main->Frame(-relief => "groove", -borderwidth => 2 )
    ->pack( -side => "top",
	    -fill => "x" );
# speed label
my $speed_label = $menu->Label( -text => "Speed: $speed",
				-relief => "flat" )
    ->pack( -side => "right" );
# blank canvas seperating $menu from $display
$main->Frame(-height => "15")
    ->pack( -fill => "x" );
# rectangle holding black box
my $display = $main->Canvas(-height => "90",
			    -width  => "640",
			    -relief => "sunken",
			    -borderwidth => 4 )
    ->pack();
# the black box
$display->createRectangle(2,2,642,92,
			  -fill => "black");
$display->fontCreate("txt_font",
		     -family => $font_type,
		     -size => $font_size);
# places $word within $display.
my $txt_tag = $display->createText(320,45,
				   -justify => "center",
				   -fill    => "$font_color",
				   -font    => "txt_font",
				   );
# blank canvas seperating $maincanvas from $scroll_view
$main->Frame( -height => "15" )->pack( -fill => "x" );
# Where the document is displayed
my $scroll_view = $main->Scrolled("TextUndo",
				  -relief =>"sunken" ,
				  -borderwidth => "2",
				  -background => "grey",
				  -width => "75",
				  -height => "20",
				  -wrap => "word")
    ->pack(-fill=>"both", -expand => 1);
# bottom Frame
my $bottom_frame = $main->Frame( -height => "20" )
    ->pack( -fill => "x" );
my $start_button = $bottom_frame->Button( -text => "Start",
					  -command => \&pause
					  )
    ->pack(-side => "left" );
my $load_button = $bottom_frame->Button(
	-text => " Load ",
	-command => \&load_button )
     ->pack(-side => "left" );
$bottom_frame->Button( -text => " Save ",
		       -command => \&ini_write )
    ->pack(-side => "right" );
my $line_info = $bottom_frame->Label(-text => "$line_index2",
				     -relief=>"ridge",
				     -width=>"5")
    ->pack(-side=>"right");
my $line_info1 = $bottom_frame->Label(-text => "Character #:")
    ->pack(-side=>"right");
my $line_info2 = $bottom_frame->Label(-text => "$line_num",
				      -relief=>"ridge",
				      -width=>"8")
    ->pack(-side=>"right");
my $line_info3 = $bottom_frame->Label(-text => "Line #:")
    ->pack(-side=>"right");
my $line_info4 = $bottom_frame->Label(-text => "$end",
				      -relief=>"ridge",
				      -width=>"8")
    ->pack(-side=>"right");
my $line_info5 = $bottom_frame->Label(-text => "Total Lines:")
    ->pack(-side=>"right");
$main->protocol('WM_DELETE_WINDOW', \&check_save);

################# MENUES #####################
########### "File" drop-down menu ############
my $menubutton1 = $menu
    ->Menubutton( -text => "File", -relief => "flat", -tearoff => 1,
		  -menuitems => [
				 ['command' => "Open File",
				  -command => \&open1],
				  # writes ini file with
				  # settings last used
				 ['command' => "Save Session",
				  -command => \&ini_write],
				 ['command' => "Exit",
				  -command => \&check_save],
				 "-",
				 ])->pack( -side => "left" );
# Display recently saved documents from dot file
foreach (0 .. 9){
    if ($recent_docs[$_][0]){
	my $tmp;
	$menubutton1->radiobutton(-label => "$recent_docs[$_][0]",
				  -variable => \$tmp,
				  -value => $_,
				  -command => sub { reload($tmp); });
    }
}
########### "Tool" drop-down menu ############
my $menubutton2 = $menu
    ->Menubutton( -text => "Tools", -relief => "flat", -tearoff => 1,
		  -menuitems => [
				 ['command' => "Configuration",
				  -command => \&configure],
				 ])
    ->pack( -side => "left" );
########### "Help" menu ###############
my $menubutton3 = $menu
    ->Menubutton( -text => "Help", -relief => "flat", -tearoff => 1,
		  -menuitems => [
				 ['command' => "About",
				  -command => \&about],
				 ])
    ->pack( -side => "left" );

#########################################################
## Keyboad Bindings
#########################################################
$main->bind("<Insert>",[sub { $status = "loaded" }]);
$main->bind("<Up>",[sub {
			$speed = $speed - 10 if $speed > 10;
			$speed_label->configure( -text => "Speed: $speed" );
		    }]);
$main->bind("<Down>",[sub {
			$speed = $speed + 10 if $speed < 510;
			$speed_label->configure( -text => "Speed: $speed" );
                      }]);
$main->bind("<Left>",[sub {
			$CHARMAX -- if $CHARMAX >3;
                      }]);
$main->bind("<Right>",[sub {
                         $CHARMAX ++ if $CHARMAX < 30;
                       }]);
$main->bind("<space>", \&pause);
$main->bind("<Pause>", \&pause);
$main->bind("<Control-Key-g>", \&gutenberg);
$main->bind("<Shift-Double-Button-1>",
	    [sub {
		 if ($status ne 'paused' && $status ne 'loaded') {
		     $status = 'double_clicked';
		 }
		 my $double_click = $scroll_view->index("sel.first");
		 my @tmp = split /\./, $double_click;
		 $line_num = $tmp[0];
		 $line = $scroll_view->get($double_click, "$tmp[0].end");
		 unless ($line_num){print "line number?\n"}
		 unless ($line){print "line?\n"}
		 if ($line =~ /\S/) {
		     @words = breakline($line);
		     unless (@words){ push @words, $line || print "words?\n"; }
		 }
		 $word_num = 0;
		 $word = $words[$word_num];
		 if ($status eq 'double_clicked') {
		     pause($double_click);
		 }
		 else {
		     print "$double_click\n";
		     $scroll_view->tagAdd( "blue",
					   "$line_num.0",
					   "$double_click"
					   );
		     $line_info->configure(-text => "$double_click");
		     $line_info2->configure(-text => "$line_num"); }
	     }]);

# Load File From Command Line
if ($file) {
    # Get working directory and set it as $file
    my $tmp = cwd;
    $tmp .= '/' . $file;
    $file = $tmp;
    $scroll_view->Load($file);
    $status = "loaded";
    $line_num = 1;
    $line_begin = "$line_num.0";
    $line_end = "$line_num.end";
    $line = $scroll_view->get($line_begin, $line_end);
    $start_button->configure(-text => "Start");
    $load_button->configure(-text => "Unload");
    @words = breakline($line);
    $display->dchars($txt_tag,0, 25);
    $end = $scroll_view->index("end");
    $line_info4->configure(-text => "$end");
}

MainLoop;

##############################################################
###################### SUBROUTINES ###########################

##############################################################
## 'About' Toplevel Canvas
##############################################################

sub about {
    my $help = $main->Toplevel();
    $help->title("About");
    $help->Label(-text => "WordFlashReader_018.pl",
		 -relief => "ridge",
		 -font => "times 15 bold"
		 )->pack(-side => "top",
			 -fill => "x",
			 -ipady => "2",
			 -padx => "5",
			 -pady => "5");

    $help->Label(-text => "(c) 2001-2002, Winship Varner")->pack(-side => "top");
    $help->Label(-text => "Licenced under GPL")->pack(-side=>"top");
    my $text = $help->Scrolled('ROText',
        	-scrollbars => 'e',
        	-wrap       => 'word',
        	-width      => 70,
        	-height     => 20,
        	-font       => $font_type,
        	-setgrid    => 0,
		-background => 'white'
    )->pack(-side => 'bottom', -expand => 1, -fill => 'both');
    $text->tagConfigure('title',
    	    -font => '-*-Helvetica-Bold-R-Normal--*-180-*-*-*-*-*-*',
    );
    my $instr = `pod2text $0`; #" comment here for text editor reasons
#    $text->insert('end', "WordFlashReader_018.pl\n\n", 'title');
    $text->insert('end', $instr);
    $help->Button(-text => "README", -command => sub {
	$text->delete('1.0', 'end');
	$text->insert('end', $instr);
    })->pack(-side => "left");

    $help->Button(-text => "GPL License", -command => sub {

	my $instr = <<'EOInstr';

Visit http://www.gnu.org/licenses/gpl.txt

EOInstr

	$text->delete('1.0', 'end');
        $text->insert('end', "GPL Licence\n", 'title');
        $text->insert('end', $instr);
    } )->pack(-side => "right");
}

###################################
## Check/Save Routine
###################################
sub check_save {
    if ($check_save) {
	exit unless $status eq "started" ||
	            $status eq "paused" ||
		    $status eq "double_clicked";
	my $save = $scroll_view->messageBox(-icon => 'warning',
	    	         	            -type => 'YesNoCancel',
					    -default => 'Yes',
					    -message => "Save Session?",
		        );
	return if $save eq 'Cancel';
	if ($save eq 'Yes') {
	    ini_write();
	}
    }
    exit;
}

#############################################
## Configuration Widget
#############################################
sub configure{
    my $config = $main->Toplevel();
    $config->title("Configure Settings");
    my @labels = ('Check_Save: 1=on, 0=off ','Font Size','Font Type', 'Font Color');
    foreach (0..$#labels){
    	$config->Label(-text=>"$labels[$_]", -anchor=>'e')->grid(-row => $_, -column => 1, -sticky=>"nsew");
    }
    $config->Entry(-width => 7, -textvariable => \$check_save)
        ->grid(-row => 0, -column => 2, -sticky=>"nsew");
    $config->Entry(-width => 7, -textvariable => \$font_size)
    	->grid(-row => 1, -column => 2, -sticky=>"nsew");
    $config->Entry(-width => 7, -textvariable => \$font_type)
    	->grid(-row => 2, -column => 2, -sticky=>"nsew");
    $config->Entry(-width => 7, -textvariable => \$font_color)
    	->grid(-row => 3, -column => 2, -sticky=>"nsew");
    $config->Button(-command => sub {
	    $display->fontConfigure("txt_font", -size => $font_size,
	    					-family => $font_type);
	    $display->itemconfigure($txt_tag, -fill => $font_color);
	    $config->destroy(); },
	    -text => "Done")->grid(-row => 4, -column => 2, -sticky=>"nsew");
}

sub gutenberg{
    $line_num = 1;
    $word_num = 0;
  FIND: {
    $line_begin = "$line_num.0";
    $line_end = "$line_num.end";
    $line = $scroll_view->get($line_begin, $line_end);
    if ($line =~ /end\*?\s?the\ssmall\sprint/i){
	print "$line\n";
	$line_num += 6;
	$scroll_view->see("$line_num.0");
    } elsif ($line_num < 500){
	$line_num += 1;
	redo FIND;
    } else {
	print "Could not find the end of the\n";
	print "Project Gutenberg legal stuff...\n";
    }
  }
}

####################################################
## The Beating Heart of this Program
####################################################
sub increment {
    # recognizes pause
    if ($status eq "paused" || $status eq "double_clicked"){
	goto &pause2;
    }
  INCREMENT:
    {
	if ( $word_num == ($#words + 1)  ) {
	    #############reseting variables############
	    $line_num += 1;
	    $word_num = 0;
	    @words = ();
	    $line = "";
	    $line_index2 = 0;
	    $line_begin = "$line_num.0";
	    $line_end = "$line_num.end";
	    if ($line_begin == $end){
		$status = "ended";
		goto &pause2;
	    }
	    $line = $scroll_view->get($line_begin, $line_end);
	    if ($line =~ /\S/){
              @words = breakline($line);
	    }
	    else {
		redo INCREMENT;
	    }
	}
    }
    $main->after($speed => \&print_word);
}

###################################################################
## ini_read: Attempts to read from an ini file. If it cannot find
## one, it creates one with default settings.
###################################################################
sub ini_read {
    # writes INI if it can't find one
    unless (open(INI_FILE, "$home/.wordflashreader.ini")) {
	$status = "writing"; # only time this is used
	ini_write();
	# attempts to re-open it
	open(INI_FILE, "$home/.wordflashreader.ini" ) ||
	    warn "Could not write dot file: $!";
    };
    while (<INI_FILE>) {
	chomp($_);
	push @configuration, $_;
    }
    close(INI_FILE);

    @recent_docs = ();
    $speed = $configuration[0]; # $speed configuration
    $font_type = $configuration[1];
    $font_size = $configuration[2];
    $font_color = $configuration[3];
    $check_save = $configuration[4];
    $home = $configuration[5];
    foreach $_ (6..16) {
	if (defined($configuration[$_])) {
	    my @tmp = split(/&&/, $configuration[$_]);
	    push @recent_docs, [ @tmp ];
	}
    }
    # sets $default_dir to last directory opened
    for ($configuration[6]){
	/(.*)\/.*/ if defined;
	$default_dir = $1 if defined;
    }
undef @configuration;
}

############################################
## Write to (or just write) the dot file
############################################
sub ini_write {
    #return if (($file eq "") && (;
    open(INI_FILE, ">$ENV{HOME}/.wordflashreader.ini\n");
    print INI_FILE "$speed\n$font_type\n$font_size\n$font_color\n$check_save\n$home\n";
    if ($file) {
	my $tmp = "0";
        foreach (@recent_docs){
	    last if !defined ($$_[0]);
	    if ($$_[0] eq $file){
	        splice @recent_docs, $tmp, 1;
	    }
	    ++$tmp;
	}
    }
    print INI_FILE "$file&&$line_num&&$word_num\n" if $file ne "";
    foreach (@recent_docs) {
	if (defined ($$_[0])) {
	    print INI_FILE "$$_[0]&&$$_[1]&&$$_[2]\n";
	}
    }

    close(INI_FILE);

    print "printed .wordflashreader.ini\n";
    # only occurs when dot file does not exist
    if ($status eq "writing"){$status = "not_loaded"}
    # otherwise ...
    else {
	ini_read();
	# makes sure the menuitems are defined before
	# they are deleted
	my $num_items = $menubutton1->menu->index('end');
	if ($num_items > 4){
	    $menubutton1->menu->delete(5, "last");
	}
	foreach (0 .. 9){
	    if ($recent_docs[$_][0]){
		my $tmp;
		$menubutton1->radiobutton(-label => "$recent_docs[$_][0]",
					  -variable => \$tmp,
					  -value => $_,
					  -command => sub { reload($tmp); });
	    }
	}
    # pauses it if someone saves it will flashing
    ($status eq "paused" || $status eq "saved" || $status eq "loaded") ? $status = "saved" : pause();
    }
}

######################################
sub load_button {
    if ($status eq "not_loaded"){
    $file = "";
    $line = $scroll_view->get("1.0", "1.end");
        if ($line ne ""){
            $load_button->configure(-text => "Unload");
	    $status = "loaded";
            @words = breakline($line);
	    $end = $scroll_view->index("end");
	}
    } elsif ($status eq "started"){
    	print "Press Button again to Unload\n";
    	pause();
    } else {
        $scroll_view->EmptyDocument;

	$load_button->configure(-text => "Load");
	$start_button->configure(-text => "Start");
	$end = 0;
	$file = "";
	@words = ();
	$word = "";
	$word_num = 0;
	$line = "";
	$line_num = 1;
	$line_begin = 0;
	$line_end = 0;
	$line_index1 = 0;
	$line_index2 = 0;
	$status = "not_loaded";
	$display->dchars($txt_tag,0, 25);
    }
}

######################################
## Open $file
######################################
# decides whether or not a file is open and uses getOpenFile to set $file
sub open1 {
    pause() if $status eq "started";
    my $file1 = "";
    $file1 = $file if defined $file;
    undef $file;
    $file = $main->getOpenFile(-initialdir=>$default_dir,
                               -filetypes =>
                               [
                                ['Text Files', ['.txt', '.CLEANED']],
                                ['All Files',  '*']
                                ],
                               -title => "Open File",
                               );
    if (defined($file)) {
	$status = "loaded";
	$scroll_view->Load($file);
	$line_num = 1;
	$line_begin = "$line_num.0";
	$line_end = "$line_num.end";
	$word = "";
	$word_num = 0;
	$line_index1 = 0;
	$line_index2 = 0;
	$display->dchars($txt_tag,0, 25);
	$line = $scroll_view->get($line_begin, $line_end);
	$start_button->configure(-text => "Start");
        @words = breakline($line);
	$load_button->configure(-text => "Unload");
	$display->dchars($txt_tag,0, 25);
	$end = $scroll_view->index("end");
	$line_info4->configure(-text => "$end");
    } else {
	$file = $file1 if $file1;
    }
}

#######################################################
## Pause Routines - Alot more happens here than pausing
#######################################################
sub pause {
    return if $status eq "not_loaded";
    if ($status eq "loaded") {
	$main->focus();
	$status = "started";
	$start_button->configure(-text => "Pause");
	goto &increment;
    }
    elsif ($status eq 'double_clicked') {
	my $double_click = shift;
	$scroll_view->tagAdd( "blue",
			      "$line_num.0",
			      "$double_click"
			    );
	$start_button->configure(-text => "Unpause");
	$line_info->configure(-text => "$double_click");
	$line_info2->configure(-text => "$line_num");
    }
    $scroll_view->see("$line_num.$line_index2");
    if ($status eq "started") {
	$status = "paused";
	$scroll_view->tagDelete("blue");
	$start_button->configure(-text => "Unpause");
	$line_info->configure(-text => "$line_index2");
	$line_info2->configure(-text => "$line_num");
    }
    elsif ($status eq "paused" || $status eq "saved") {
	$main->focus();
	$scroll_view->tagDelete("blue");
	$status = "started";
	$start_button->configure(-text => "Pause");
	&pause2();
    }
}

sub pause2 {
    if ($status ne "started") {
	if ($status ne "double_clicked") {
	    $scroll_view->tagConfigure( "blue",
					-foreground => "blue",
					);
	    $scroll_view->tagAdd( "blue",
				  "$line_num.0",
				  "$line_num.$line_index2"
				  );
	}
	elsif ($status eq "double_clicked") {
	    $status = "paused";
	}
    }
    if ($status eq "started") {
	&increment();
    }
    if ($status eq "ended") {
	$start_button->configure(-text => "Start");
	$status = "loaded";
	$line_num = 1;
	$word_num = 0;
	$line_begin = "$line_num.0";
	$line_end = "$line_num.end";
	$line = $scroll_view->get($line_begin, $line_end);
        @words = breakline($line);
	print "__eof__\n";
    }
}
# The pauses after punctuation
sub print_blank1 {
    $display->dchars($txt_tag,0, 25);
    if ( $punct eq "long_pause" ) {
	$punct = "";
	# longer display of blank
	my $speed2 = (2 * ($speed + 100));
	$main->after($speed2 => \&increment);
    }
    elsif ( $punct eq "short_pause" ) {
	$punct = "";
	# shorter than end of sentence punctuation
	my $speed3 = (2 * ($speed + 50));
	$main->after($speed3 => \&increment);
    }
}
###########################################################
## Display Words and Send Punctuation to sub punctuation{}
###########################################################
sub print_word {
    if($status ne "double_click") {
	$word = $words[$word_num];
	chomp($word);
	$line_index1 = $line_index2;
	$line_index2 = $line_index1 + length( $word ) + 1;
    }
    # find period, question mark, or exclamation
    # (also each in the context of quotes), and semi-colon
    if ( $word =~ m/\.$/ ||
	 $word =~ m/\?$/ ||
	 $word =~ m/\!$/ ||
	 $word =~ m/\;$/ ||
	 $word =~ m/\.\"$/ ||
	 $word =~ m/\?\"$/ ||
	 $word =~ m/\!\"$/ ) {
	$punct = "long_pause";
	$display->dchars($txt_tag,0, 25);
	$display->insert($txt_tag,0,$word);
	$word_num += 1;
	&punctuation();
    }
    # find semi-colon, comma, colon
    elsif ( $word =~ m/\,$/ ||
	    $word =~ m/\:$/ ||
	    $word =~ m/\,\"$/ ) {
	$punct = "short_pause";
	$display->dchars($txt_tag,0, 25);
	$display->insert($txt_tag,0,$word);
	$word_num += 1;
	&punctuation();
    }
    # no punctuation
    else {
	# slight pauses *on* long words: 10 letters or more
#	if ((length ( $word )) >= 9){
#	    $display->dchars($txt_tag,0, 25);
#	    $display->insert($txt_tag,0,$word);
#	    $word_num += 1;
#	    $main->after($speed => \&increment);
#	} else {
	    $display->dchars($txt_tag,0, 25);
	    $display->insert($txt_tag,0,$word);
	    $word_num += 1;
	    &increment();
#	}
    }
}

sub punctuation {
    $main->after($speed => \&print_blank1);
}

##############################################
## Reload $scrolled_view and reset variables
##############################################
sub reload {
    pause() if $status eq "started";
    my $i = $_[0];
    $file = $recent_docs[$i][0];
    # Tests and makes sure the file is actually
    # there before trying to load it. If it isn't then
    # delete it from @recent_docs
    if (open (TEST, $file)) {
     	close TEST || warn "file not open";
    } else {
    	$file = "";
	$scroll_view->EmptyDocument;
	print "FILE HAS MOVED\n";
	splice @recent_docs, $i, 1;
	ini_write();
	return;
    }
    $scroll_view->Load($file);
    $line_num = $recent_docs[$i][1];
    $line_begin = "$line_num.0";
    $line_end = "$line_num.end";
    $line = $scroll_view->get($line_begin, $line_end);
    $start_button->configure(-text => "Start");
    @words = breakline($line);
    $word_num = $recent_docs[$i][2];
    $display->dchars($txt_tag,0, 25);
    $load_button->configure(-text => "Unload");
    $line_index2 = 0;
    if ($word_num > 0) {
    	foreach (0 .. ($word_num - 1)) {
	    $line_index2 = $line_index2 + length( $words[$_] ) + 1;
	}
    }

    $end = $scroll_view->index("end");
    $line_info4->configure(-text => "$end");
    $line_info2->configure(-text => "$line_num");
    $status = "loaded";
    $scroll_view->see("$line_num.$line_index2");
    $scroll_view->tagConfigure( "blue",
				-foreground => "blue",
				);
    $scroll_view->tagAdd( "blue",
			  "$line_num.0",
			  "$line_num.$line_index2"
			  );
}



sub breakline{
  my ($paragraph) = @_;
  my @words = ();

  if (!defined $paragraph) {
    return @words;
  }

  chomp $paragraph;
  


  my $CHARCT= int($CHARMAX - ($CHARMAX/4));
  my $CHARCT2= $CHARMAX;
  my $CHARCT3= int($CHARMAX + ($CHARMAX/4));
  
  $paragraph = $linesave." ".$paragraph;

  while (length ($paragraph)>0) {
    my ($chunk) = $paragraph=~/(^.{$CHARCT}[^- ]*-*)/;
    my ($chunk2) = $paragraph=~/(^.{$CHARCT2}[^- ]*-*)/;
    my ($chunk3) = $paragraph=~/(^.{$CHARCT3}[^- ]*-*)/;
    if (!defined $chunk3) {
      $chunk = $paragraph;
      $chunk2 = $paragraph;
      $chunk3 = $paragraph;
      push @words, $paragraph;
      $paragraph = "";
      return @words;
    }
    
    if (abs(length($chunk2)-(($CHARCT2+$CHARCT+$CHARCT3)/3)) <
        abs(length($chunk)-(($CHARCT2+$CHARCT+$CHARCT3)/3))) {
      $chunk = $chunk2;
    }
    
    if (abs(length($chunk3)-(($CHARCT2+$CHARCT+$CHARCT3)/3)) <
        abs(length($chunk)-(($CHARCT2+$CHARCT+$CHARCT3)/3))) {
      $chunk = $chunk3;
    }
    

#    if ((length($chunk)>=$CHARCT) || (length($chunk) == length($paragraph)) ) {
      push @words, $chunk;
#      $linesave = "";
#    } else {
#      $linesave = $chunk
#    }

    $chunk =~ s/\?/./g;
    $chunk =~ s/\]/./g;
    $chunk =~ s/\(/./g;
    $chunk =~ s/\)/./g;
    $chunk =~ s/\[/./g;
    
    $paragraph =~ s/ ?$chunk ?//;
    print "$chunk\n";
  }

  return @words;
}

################################################################
#################################################################
__END__

=pod


=head1 NAME

WordFlashReader_018.pl

=head1 AUTHOR

Winship Varner

=head1 EMAIL

winshipvarner@yahoo.com

=head1 LICENSE

This program is free software; you can redistribute it and/or
modify it in accordance with the GNU General Public Licence.

Copyright: (c) 2001-2002 Winship Varner

=head1 INTRODUCTION

Welcome to my WordFlashReader program! It is
intended for anyone who has an electronic text or book
they wish to read (you can find many in on the Project
Gutenberg website: http://www.gutenberg.net). It will
flash each word of the text/book sequentially, pausing
at the punctuation. After a little practice, you may
learn to read at multiple times the speed that you
normally do. Look under FEATURES below to see how to
use the program. This program does not work well under
Windows. Any suggestions about how to fix this are welcome.

=head1 INSTALLATION AND USAGE

In order to get it working, you need to have Perl
working on your computer. You will also need to install
the Tk module. Both can be downloaded from
http://www.perl.com/CPAN.

Make sure to make WordFlashReader executable (chmod 755)
if your in Linux. Once you have done this, you can load
a file from the prompt:

 % ./WordFlashReader_018.pl myfile &

Or better yet, alias WordFlashReader in your .bashrc

=head1 THE FORMAT OF THE FILE THIS PROGRAM WILL READ

WordFlashReader will read any text file. You
may need to clean it up a bit before opening it, i.e.,
confusing symbols or (especially) extra blank spaces.
To help cleanup the latter, I have included a simple
perl script "space_cleanup.pl", that searches a txt
file for any occurances of two or more blank spaces
and replaces them with only one -- that is, in a new
file (the_file.CLEANED). To see what was changed, type
at the command prompt:

 % diff file.txt file.txt.CLEANED


=head1 FEATURES

After opening a file, it should appear in the scroll
window. To start, either press the "Start" button (or
hit the "Pause" button), and the words will start
flashing at you. Use the up arrow to speed up the
flashing and the down arrow to slow it down. The
Spacebar (or the "Pause" button) will pause the
flashing causing the scroll view window to scroll to
the place where you have paused, highlighting the line
and word.

You can configure the font type, size and color by
clicking on the Tools->Configuration menuitem. Also
you can turn off the dialog box that appears as a
reminder before exiting an unsaved file by changing
the value to "0".

If you have finished reading and want to save your spot,
click the "Save" button, and the next time you open the
program your document will appear in the history list
found in the "File" menu. It should open it where you
left off.

Also, if you want to start at a particular place (rather
than the beginning), go to that place, and double click
the left mouse button while holding down the shift key.
Also if you have downloaded a gutenberg txt file, you can
have WordFlashReader skip over all the legal stuff by
pressing the Control button and "g" at the same time
(its not guaranteed to work all the time, but it seems
to work most of the time).

=head1 CHANGES

=over 4

=item *

There have been quite a number of changes, enhancements,
and bug fixes since the previous release.

=back

=head1 TODO

=over 4

=item *

Allow configuration of default directory to load from.

=item *

Increase font size in the text pad.

=back

=head1 BUGS

=over 4

=item *

There is a bug in Tk 800.023 in: FBox.pm. Update to
Tk 800.024

=item *

There is a bug somewhere in the shift-double click
method, but I am not sure exactly how to reproduce it.

=back

Submit corrections or bug reports to the email address
mentioned above.


=cut
